import fast_stark_crypto


def test_well_known_transfer_hash() -> None:
    expected_hash = int("2453233552426028319905576552240018623512915507691922332978444357299842334607")
    actual_hash = fast_stark_crypto.get_transfer_msg_hash(
        recipient_position_id=1,
        sender_position_id=2,
        collateral_id=3,
        amount=4,
        expiration=5,
        salt=6,
        user_public_key=int("2629686405885377265612250192330550814166101744721025672593857097107510831364"),
        domain_name="Perpetuals",
        domain_version="v0",
        domain_chain_id="SN_SEPOLIA",
        domain_revision="1",
    )
    assert actual_hash == expected_hash, "Hashes do not match for get_transfer_msg_hash"



import fast_stark_crypto


def test_well_known_order_hash() -> None:
    hash_from_rust = fast_stark_crypto.get_order_msg_hash(
        position_id=100,
        base_asset_id=2,
        base_amount=100,
        quote_asset_id=1,
        quote_amount=-156,
        fee_asset_id=1,
        fee_amount=74,
        expiration=100,
        salt=123,
        user_public_key=int("0x5d05989e9302dcebc74e241001e3e3ac3f4402ccf2f8e6f74b034b07ad6a904", 16),
        domain_name="Perpetuals",
        domain_version="v0",
        domain_chain_id="SN_SEPOLIA",
        domain_revision="1",
    )

    assert hash_from_rust == int("0x4de4c009e0d0c5a70a7da0e2039fb2b99f376d53496f89d9f437e736add6b48", 16)


def test_sdk_buy_order_value() -> None:
    synth_id = "0x4254432d3600000000000000000000"
    collateral_id = "0x31857064564ed0ff978e687456963cba09c2c6985d8f9300a1de4962fafa054"
    fee_id = collateral_id
    fee_amount = 21723
    base_amount = 1000
    collateral_amount = -43445117
    hash_from_rust = fast_stark_crypto.get_order_msg_hash(
        position_id=10002,
        base_asset_id=int(synth_id, 16),
        base_amount=base_amount,
        quote_asset_id=int(collateral_id, 16),
        quote_amount=collateral_amount,
        fee_asset_id=int(fee_id, 16),
        fee_amount=fee_amount,
        expiration=1706836137,
        salt=1473459052,
        user_public_key=int("0x61c5e7e8339b7d56f197f54ea91b776776690e3232313de0f2ecbd0ef76f466", 16),
        domain_name="Perpetuals",
        domain_version="v0",
        domain_chain_id="SN_SEPOLIA",
        domain_revision="1",
    )

    assert hash_from_rust == int("0x58454e78c25644cbcab59444736d573f169fb0996dafe1900a05e2ac50567f0", 16)



from fast_stark_crypto.fast_stark_crypto import (
    rs_get_public_key,
    rs_compute_pedersen_hash,
    rs_sign_message,
    rs_verify_signature,
    rs_get_order_msg,
    rs_get_transfer_msg,
    rs_generate_keypair_from_eth_signature,
    rs_get_withdrawal_hash,
)


def get_public_key(private_key: int) -> int:
    return int(rs_get_public_key(hex(private_key)))


def pedersen_hash(first: int, second: int) -> int:
    return int(rs_compute_pedersen_hash(hex(first), hex(second)))


def sign(private_key: int, msg_hash: int) -> tuple[int, int]:
    (r, s) = rs_sign_message(hex(private_key), hex(msg_hash))
    return (int(r), int(s))


def verify(public_key: int, msg_hash: int, r: int, s: int) -> bool:
    return bool(rs_verify_signature(hex(public_key), hex(msg_hash), hex(r), hex(s)) == True)


def generate_keypair_from_eth_signature(
    eth_signature: str,
) -> tuple[int, int]:
    (priv, pub) = rs_generate_keypair_from_eth_signature(eth_signature)
    return (int(priv, 16), int(pub, 16))


def get_order_msg_hash(
    position_id: int,
    base_asset_id: int,
    base_amount: int,
    quote_asset_id: int,
    quote_amount: int,
    fee_asset_id: int,
    fee_amount: int,
    expiration: int,
    salt: int,
    user_public_key: int,
    domain_name: str,
    domain_version: str,
    domain_chain_id: str,
    domain_revision: str,
) -> int:
    return int(
        rs_get_order_msg(
            str(position_id),
            hex(base_asset_id),
            str(base_amount),
            hex(quote_asset_id),
            str(quote_amount),
            hex(fee_asset_id),
            str(fee_amount),
            str(expiration),
            str(salt),
            hex(user_public_key),
            domain_name,
            domain_version,
            domain_chain_id,
            domain_revision,
        ),
        16,
    )


def get_transfer_msg_hash(
    recipient_position_id: int,
    sender_position_id: int,
    collateral_id: int,
    amount: int,
    expiration: int,
    salt: int,
    user_public_key: int,
    domain_name: str,
    domain_version: str,
    domain_chain_id: str,
    domain_revision: str,
) -> int:
    return int(
        rs_get_transfer_msg(
            str(recipient_position_id),
            str(sender_position_id),
            hex(collateral_id),
            str(amount),
            str(expiration),
            str(salt),
            hex(user_public_key),
            domain_name,
            domain_version,
            domain_chain_id,
            domain_revision,
        ),
        16,
    )


def get_withdrawal_msg_hash(
    recipient_hex: str,
    position_id: int,
    collateral_id: int,
    amount: int,
    expiration: int,
    salt: int,
    user_public_key: int,
    domain_name: str,
    domain_version: str,
    domain_chain_id: str,
    domain_revision: str,
) -> int:
    return int(
        rs_get_withdrawal_hash(
            recipient_hex,
            str(position_id),
            hex(collateral_id),
            str(amount),
            str(expiration),
            str(salt),
            hex(user_public_key),
            domain_name,
            domain_version,
            domain_chain_id,
            domain_revision,
        ),
        16,
    )



import fast_stark_crypto


def test_well_known_keypair_generation() -> None:
    (priv, pub) = fast_stark_crypto.generate_keypair_from_eth_signature(
        eth_signature="0x9ef64d5936681edf44b4a7ad713f3bc24065d4039562af03fccf6a08d6996eab367df11439169b417b6a6d8ce81d409edb022597ce193916757c7d5d9cbf97301c"
    )

    assert priv == int("0x7dbb2c8651cc40e1d0d60b45eb52039f317a8aa82798bda52eee272136c0c44", 16)
    assert pub == int("0x78298687996aff29a0bbcb994e1305db082d084f85ec38bb78c41e6787740ec", 16)



def rs_get_public_key(private_key: str) -> str: ...
def rs_pedersen_hash(left: str, right: str) -> str: ...
def rs_sign(msg_hash: str, private_key: str, k: str) -> tuple[str, str]: ...
def rs_verify(msg_hash: str, sig_r: str, sig_s: str, public_key: str) -> bool: ...
